@@ -1,6 +1,7 @@
 #!/usr/bin/env python3
 import os
 import sys
+import gzip
 import glob
 import json
 import argparse
@@ -47,22 +48,23 @@ def parse_args(args):
                      help='DiamondDB reference file. Default: GUNC_DB envvar',
                      default=os.environ.get('GUNC_DB'),
                      metavar='')
-    run_group.add_argument('-i', '--input_fna',
-                           help='Input file in FASTA fna format.',
+    run_group.add_argument('-i', '--input_fasta',
+                           help='Input file in FASTA format.',
                            metavar='')
     run_group.add_argument('-f', '--input_file',
-                           help='File with paths to FASTA fna format files.',
+                           help='File with paths to FASTA format files.',
                            metavar='')
     run_group.add_argument('-d', '--input_dir',
-                           help='Input dir with files in FASTA fna format.',
+                           help='Input dir with files in FASTA format.',
                            metavar='')
     run.add_argument('-e', '--file_suffix',
                      help='Suffix of files in input_dir. Default: .fa',
                      default='.fa',
                      metavar='')
-    run_group.add_argument('-g', '--gene_calls',
-                           help='Input genecalls FASTA faa format.',
-                           metavar='')
+    run.add_argument('-g', '--gene_calls',
+                     help='Input files are FASTA faa format. Default: False',
+                     action='store_true',
+                     default=False)
     run.add_argument('-t', '--threads',
                      help='number of CPU threads. Default: 4',
                      default='4',
@@ -121,12 +123,16 @@ def parse_args(args):
     download_db.add_argument('path',
                              help='Download database to given direcory.',
                              metavar='dest_path')
+    download_db.add_argument('-db', '--database',
+                             help='Which db to download. progenomes,gtdb Default: progenomes',
+                             default='progenomes',
+                             metavar='')
     merge_checkm.add_argument('-g', '--gunc_file',
                               help='Path of GUNC.maxCSS_level.tsv file.',
                               required=True,
                               metavar='')
     merge_checkm.add_argument('-c', '--checkm_file',
-                              help='CheckM output file',
+                              help='CheckM qa output file',
                               required=True,
                               metavar='')
     merge_checkm.add_argument('-o', '--out_dir',
@@ -178,7 +184,15 @@ def get_files_in_dir_with_suffix(directory, suffix):
     return files
 
 
-def merge_genecalls(genecall_files, out_dir):
+def openfile(filename):
+    """Open file whether gzipped or not."""
+    if filename.endswith('.gz'):
+        return gzip.open(filename, 'r')
+    else:
+        return open(filename, 'r')
+
+
+def merge_genecalls(genecall_files, out_dir, file_suffix):
     """Merge genecall files.
 
     Merges fastas together to run diamond more efficiently.
@@ -188,6 +202,7 @@ def merge_genecalls(genecall_files, out_dir):
     Arguments:
         genecall_files (list): Paths of genecall fastas to merge
         out_dir (str): Directory to put the merged file
+        file_suffix (str): Suffix of input files
 
     Returns:
         str: path of the merged file
@@ -196,18 +211,20 @@ def merge_genecalls(genecall_files, out_dir):
     with open(merged_outfile, 'w') as ofile:
         for file in genecall_files:
             if os.path.isfile(file):
-                with open(file, 'r') as infile:
+                with openfile(file) as infile:
                     genome_name = os.path.basename(file).replace('.genecalls.faa',
                                                                  '')
+                    if genome_name.endswith(file_suffix):
+                        genome_name = genome_name[:-len(file_suffix)]
                     for line in infile:
                         if line.startswith('>'):
                             contig_name = line.split(' ')[0]
                             line = f'{contig_name}_-_{genome_name}\n'
-                        ofile.write(line)
+                        ofile.write(f'{line.strip()}\n')
     return merged_outfile
 
 
-def split_diamond_output(diamond_outfile, out_dir):
+def split_diamond_output(diamond_outfile, out_dir, db):
     """Split diamond output into per-sample files.
 
     Separate diamond output file into the constituent sample files.
@@ -216,6 +233,7 @@ def split_diamond_output(diamond_outfile, out_dir):
     Arguments:
         diamond_outfile (str): path to the diamond file to be split
         out_dir (str): Directory in which to put the split files.
+        db (str): Which db is used: progenomes or gtdb
 
     Returns:
         list: Of the split file paths
@@ -228,7 +246,7 @@ def split_diamond_output(diamond_outfile, out_dir):
             line = line.replace(f'_-_{genome_name}', '')
             output[genome_name] = output.get(genome_name, '') + line
     for genome_name in output:
-        outfile = os.path.join(out_dir, f'{genome_name}.diamond.out')
+        outfile = os.path.join(out_dir, f'{genome_name}.diamond.{db}.out')
         outfiles.append(outfile)
         with open(outfile, 'w') as ofile:
             ofile.write(output[genome_name])
@@ -243,16 +261,29 @@ def write_json(data, filename):
 
 def get_paths_from_file(input_file):
     """Extract paths from a text file."""
+    if input_file.endswith('.gz'):
+        sys.exit('[ERROR] Input should be list of filepaths: use -i instead?.')
     with open(input_file, 'r') as f:
         paths = f.readlines()
+    if paths[0].startswith('>'):
+        sys.exit('[ERROR] Input should be list of filepaths: use -i instead?.')
     return [path.strip() for path in paths]
 
 
-def run_from_gene_calls(gene_calls):
-    """Get genecalls and genecount from gene fasta."""
-    input_basename = os.path.basename(gene_calls)
-    genes_called = {input_basename: record_count(gene_calls)}
-    return genes_called, gene_calls
+def run_from_gene_calls(faas, out_dir, file_suffix):
+    """Prepare genecalls for running diamond."""
+    print(f'[START] {datetime.now().strftime("%H:%M:%S")} Merging genecalls..',
+          flush=True)
+    genes_called = {}
+    for faa in faas:
+        basename = os.path.basename(faa).split(file_suffix)[0]
+        if basename.endswith('.genecalls'):
+            basename = basename.split('.genecalls')[0]
+        genes_called[basename] = record_count(faa)
+    diamond_inputfile = merge_genecalls(faas, out_dir, file_suffix)
+    print(f'[END]   {datetime.now().strftime("%H:%M:%S")} Finished Merging..',
+          flush=True)
+    return genes_called, diamond_inputfile
 
 
 def run_from_fnas(fnas, out_dir, file_suffix, threads):
@@ -286,7 +317,7 @@ def run_from_fnas(fnas, out_dir, file_suffix, threads):
         basename = os.path.basename(fna).split(file_suffix)[0]
         genes_called[basename] = record_count(prodigal_outfile)
         genecall_files.append(prodigal_outfile)
-    diamond_inputfile = merge_genecalls(genecall_files, out_dir)
+    diamond_inputfile = merge_genecalls(genecall_files, out_dir, file_suffix)
     print(f'[END]   {datetime.now().strftime("%H:%M:%S")} Finished Prodigal..',
           flush=True)
     return genes_called, diamond_inputfile
@@ -297,7 +328,7 @@ def run_prodigal(prodigal_info):
     external_tools.prodigal(fna, prodigal_outfile)
 
 
-def run_diamond(infile, threads, temp_dir, db_file, out_dir):
+def run_diamond(infile, threads, temp_dir, db_file, out_dir, db):
     """Run diamond and split ouput.
 
     Runs diamond on infile and if needed splits the constitiuent samples.
@@ -308,19 +339,20 @@ def run_diamond(infile, threads, temp_dir, db_file, out_dir):
         temp_dir (str): Path of tempdir for diamond running.
         db_file (str): Path to diamond database file (GUNC_DB).
         out_dir (str): Path of directory in which to put the output files.
+        db (str): Which db is used: progenomes or gtdb
 
     Returns:
         list: Of diamond output files.
     """
     print(f'[START] {datetime.now().strftime("%H:%M:%S")} Running Diamond..',
           flush=True)
-    outfile = os.path.join(out_dir, f'{os.path.basename(infile)}.diamond.out')
+    outfile = os.path.join(out_dir, f'{os.path.basename(infile)}.diamond.{db}.out')
     external_tools.diamond(infile, threads, temp_dir, db_file, outfile)
 
     if infile.endswith('merged.genecalls.faa'):
         out_dir = os.path.join(out_dir, 'diamond_output')
         create_dir(out_dir)
-        diamond_outfiles = split_diamond_output(outfile, out_dir)
+        diamond_outfiles = split_diamond_output(outfile, out_dir, db)
         os.remove(outfile)
         os.remove(infile)
     else:
@@ -331,7 +363,7 @@ def run_diamond(infile, threads, temp_dir, db_file, out_dir):
 
 
 def run_gunc(diamond_outfiles, genes_called, out_dir, sensitive,
-             detailed_output, min_mapped_genes, use_species_level):
+             detailed_output, db, min_mapped_genes, use_species_level):
     """Call GUNC scores on diamond output files.
 
     Outputs a pandas.DataFrame with one line per inputfile
@@ -344,6 +376,7 @@ def run_gunc(diamond_outfiles, genes_called, out_dir, sensitive,
         out_dir (str): Path to output directory.
         sensitive (bool): Run with high sensitivity.
         detailed_output (bool): Output scores for every taxlevel.
+        db (str): Which db to use: progenomes or gtdb
         min_mapped_genes (int): Minimum number of mapped genes
                                 at which to calculate scores
         use_species_level (bool): Allow species level to be picked as maxCSS
@@ -355,15 +388,15 @@ def run_gunc(diamond_outfiles, genes_called, out_dir, sensitive,
           flush=True)
     gunc_output = []
     for diamond_file in diamond_outfiles:
-        basename = os.path.basename(diamond_file).replace('.diamond.out', '')
+        basename = os.path.basename(diamond_file).split('.diamond.')[0]
         gene_call_count = genes_called[basename]
         detailed, single = chim_score(diamond_file, gene_call_count,
                                       sensitive, min_mapped_genes,
-                                      use_species_level)
+                                      use_species_level, db)
         if detailed_output:
             detailed_gunc_out_dir = os.path.join(out_dir, 'gunc_output')
             detailed_gunc_out_file = os.path.join(detailed_gunc_out_dir,
-                                                  f'{basename}.all_levels.tsv')
+                                                  f'{basename}.{db}.all_levels.tsv')
             create_dir(detailed_gunc_out_dir)
             detailed.to_csv(detailed_gunc_out_file,
                             index=False,
@@ -372,7 +405,7 @@ def run_gunc(diamond_outfiles, genes_called, out_dir, sensitive,
         gunc_output.append(single)
     print(f'[END]   {datetime.now().strftime("%H:%M:%S")} Finished scoring..',
           flush=True)
-    return pd.concat(gunc_output)
+    return pd.concat(gunc_output).sort_values('genome')
 
 
 def check_for_duplicate_filenames(fnas, file_suffix):
@@ -407,20 +440,23 @@ def run(args):
         sys.exit(f'[ERROR] Output Directory {args.out_dir} doesnt exist.')
 
     if args.input_dir:
-        fnas = get_files_in_dir_with_suffix(args.input_dir, args.file_suffix)
+        fastas = get_files_in_dir_with_suffix(args.input_dir, args.file_suffix)
     elif args.input_file:
-        fnas = get_paths_from_file(args.input_file)
-        fnas = remove_missing_fnas(fnas)
-    elif args.input_fna:
-        fnas = [args.input_fna]
+        fastas = get_paths_from_file(args.input_file)
+        fastas = remove_missing_fnas(fastas)
+    elif args.input_fasta:
+        fastas = [args.input_fasta]
+
+    check_for_duplicate_filenames(fastas, args.file_suffix)
 
     if args.gene_calls:
         gene_calls_out_dir = args.out_dir
-        genes_called, diamond_input = run_from_gene_calls(args.gene_calls)
+        genes_called, diamond_input = run_from_gene_calls(fastas,
+                                                          gene_calls_out_dir,
+                                                          args.file_suffix)
     else:
-        check_for_duplicate_filenames(fnas, args.file_suffix)
         gene_calls_out_dir = os.path.join(args.out_dir, 'gene_calls')
-        genes_called, diamond_input = run_from_fnas(fnas,
+        genes_called, diamond_input = run_from_fnas(fastas,
                                                     gene_calls_out_dir,
                                                     args.file_suffix,
                                                     args.threads)
@@ -428,16 +464,21 @@ def run(args):
     genes_called_outfile = os.path.join(gene_calls_out_dir, 'gene_counts.json')
     write_json(genes_called, genes_called_outfile)
 
-    diamond_outfiles = run_diamond(diamond_input, args.threads,
-                                   args.temp_dir, args.db_file, args.out_dir)
-    if not args.gene_calls and len(diamond_outfiles) != len(fnas):
+    if 'gtdb' in os.path.basename(args.db_file):
+        db = 'gtdb_95'
+    else:
+        db = 'progenomes_2.1'
+
+    diamond_outfiles = run_diamond(diamond_input, args.threads, args.temp_dir,
+                                   args.db_file, args.out_dir, db)
+    if not args.gene_calls and len(diamond_outfiles) != len(fastas):
         diamond_outfiles = add_empty_diamond_output(args.out_dir,
-                                                    fnas,
+                                                    fastas,
                                                     args.file_suffix)
     gunc_output = run_gunc(diamond_outfiles, genes_called, args.out_dir,
-                           args.sensitive, args.detailed_output,
+                           args.sensitive, args.detailed_output, db,
                            args.min_mapped_genes, args.use_species_level)
-    gunc_out_file = os.path.join(args.out_dir, 'GUNC.maxCSS_level.tsv')
+    gunc_out_file = os.path.join(args.out_dir, f'GUNC.{db}.maxCSS_level.tsv')
     gunc_output.to_csv(gunc_out_file, index=False, sep='\t', na_rep='nan')
 
 
@@ -492,7 +533,7 @@ def get_genecount_from_gunc_output(gene_counts_file, basename):
 
 def plot(args):
     """Run visualisation function."""
-    basename = os.path.basename(args.diamond_file).replace('.diamond.out', '')
+    basename = os.path.basename(args.diamond_file).split('.diamond.')[0]
     genes_called = get_genecount_from_gunc_output(get_gene_count_file(args),
                                                   basename)
     viz_html = vis.create_viz_from_diamond_file(args.diamond_file,
@@ -517,11 +558,15 @@ def main():
     start_time = datetime.now()
     print(f'[START] {start_time.strftime("%H:%M:%S %Y-%m-%d")}')
     if args.cmd == 'download_db':
-        gunc_database.get_db(args.path)
+        gunc_database.get_db(args.path, args.database)
     if args.cmd == 'run':
         start_checks()
         if not args.db_file:
             sys.exit('[WARNING] database_file argument missing.')
+        else:
+            if not os.path.isfile(args.db_file):
+                sys.exit('[WARNING] database_file not found.')
+
         run(args)
     if args.cmd == 'plot':
         plot(args)
